@@ -163,7 +163,7 @@ emtrends = function(object, specs, var, delta.var=.001*rng,
     
     rgargs$options$just.data = TRUE
     data = do.call("ref_grid", c(rgargs))
-    rgargs$options$just.data = NULL
+    rgargs$options$just.data = rgargs$data = NULL
     
     x = data[[var]]
     fcn = NULL   # differential
@@ -192,15 +192,21 @@ emtrends = function(object, specs, var, delta.var=.001*rng,
     rgargs$options = c(rgargs$options, list(var = var, delts = delts))
     bigRG = do.call("ref_grid", c(rgargs, data = quote(data)))
     
-    ### var.subs is list of indexes for each value of delts
+    ### create var.subs -- list of indexes for each value of delts
+    ## Since var was created as slowest-varying, only multivariate factors will vary slower
     gdim = nrow(bigRG@grid) / length(delts)
     mdim = 1
-    for (v in bigRG@roles$multresp) 
-        mdim = mdim * length(bigRG@levels[[v]])
+    # added code: only consider mult levels that come AFTER var (post-processing could affect sort order)
+    levnms = names(bigRG@levels)
+    vi = which(levnms == var)
+    for (v in bigRG@roles$multresp)
+        if (which(levnms == v) > vi)
+            mdim = mdim * length(bigRG@levels[[v]])
     arr = array(seq_len(nrow(bigRG@grid)), c(gdim / mdim, length(delts), mdim))
     var.subs = lapply(seq_along(delts), function(i) as.numeric(arr[,i,]))
     
     RG = orig.rg = bigRG[var.subs[[idx.base]]]  # the subset that corresponds to reference values
+
     row.names(RG@grid) = seq_along(RG@grid[[1]])
 
     linfct = lapply(seq_along(delts), function(i) bigRG@linfct[var.subs[[i]], , drop = FALSE])
@@ -247,7 +253,8 @@ emtrends = function(object, specs, var, delta.var=.001*rng,
     
     # args for emmeans calls
     args = list(object = NULL, specs = specs, ...)
-    args$at = args$cov.reduce = args$mult.levs = args$vcov. = args$data = args$trend = args$transform = NULL
+    args$at = args$cov.reduce = args$mult.levs = args$vcov. = args$data = args$trend =
+              args$regrid = args$tran = NULL
     if(max.degree > 1) {
         chk = union(all.vars(specs), args$by)
         if (!("degree" %in% chk))
