@@ -36,8 +36,8 @@
 #' diverted to \code{\link{hpd.summary}}.
 #' 
 #' \code{confint.emmGrid} is equivalent to \code{summary.emmGrid with 
-#' infer = c(TRUE, FALSE)}. When called with \code{joint = FALSE}, \code{test.emmGrid}
-#' is equivalent to \code{summary.emmGrid} with \code{infer = c(FALSE, TRUE)}. 
+#' infer = c(TRUE, FALSE)}. The function \code{test.emmGrid}, when called with 
+#' \code{joint = FALSE}, is equivalent to \code{summary.emmGrid} with \code{infer = c(FALSE, TRUE)}. 
 #' 
 #' With \code{joint = TRUE}, \code{test.emmGrid} calculates the Wald test of the
 #' hypothesis \code{linfct \%*\% bhat = null}, where \code{linfct} and
@@ -46,7 +46,8 @@
 #' non-estimable. It is permissible for the rows of \code{linfct} to be linearly
 #' dependent, as long as \code{null == 0}, in which case a reduced set of 
 #' contrasts is tested. Linear dependence and nonzero \code{null} cause an 
-#' error.
+#' error. The returned object has an aditional \code{"est.fcns"} attribute, which
+#' is a list of the linear functions associated with the joint test.
 #'
 #' @param object An object of class \code{"emmGrid"} (see \link{emmGrid-class})
 #' @param infer A vector of one or two logical values. The first determines
@@ -61,6 +62,9 @@
 #' @param by Character name(s) of variables to use for grouping into separate 
 #'   tables. This affects the family of tests considered in adjusted \emph{P}
 #'   values. 
+#' @param cross.adjust Character: \eqn{p}-value adjustment method to 
+#'   additionally apply \emph{across} 
+#'   the \code{by} groups. See the section on P-value adjustments for details.
 #' @param type Character: type of prediction desired. This only has an effect if
 #'   there is a known transformation or link function. \code{"response"} 
 #'   specifies that the inverse transformation be applied. \code{"mu"} (or 
@@ -217,6 +221,28 @@
 #'   \emph{Warning:} Non-estimable cases are \emph{included} in the family to which adjustments
 #'   are applied. You may wish to subset the object using the \code{[]} operator
 #'   to work around this problem.
+#'   
+#'   The \code{cross.adjust} argument is a way of specifying a multiplicity
+#'   adjustment across the \code{by} groups (otherwise by default, each group is
+#'   treated as a separate family in regards to multiplicity adjustments). It
+#'   applies only to \eqn{p} values. Valid options are one of the
+#'   \code{p.adjust.methods} or \code{"sidak"}. This argument is ignored unless
+#'   it is other than \code{"none"}, there is more than one \code{by} group, and
+#'   they are all the same size. Under those conditions, we first use
+#'   \code{adjust} to determine the within-group adjusted \eqn{p} values.
+#'   Imagine each group's adjusted \eqn{p} values arranged in side-by-side
+#'   columns, thus forming a matrix with the number of columns equal to the
+#'   number of \code{by} groups. Then we use the \code{cross.adjust} method to
+#'   further adjust the adjusted \eqn{p} values in each row of this matrix. Note
+#'   that an \emph{overall} Bonferroni (or Sidak) adjustment is obtainable by
+#'   specifying \emph{both} \code{adjust} and \code{cross.adjust} as
+#'   \code{"bonferroni"} (or \code{"sidak"}). However, less conservative (but
+#'   yet conservative) overall adjustments are available when it is possible to
+#'   use an \dQuote{exact} within-group method (e.g., \code{adjust = "tukey"}
+#'   for pairwise comparisons) and \code{cross.adjust} as a conservative
+#'   adjustment. [\code{cross.adjust} methods other than \code{"none"}, 
+#'   \code{"bonferroni"}, or \code{"sidak"} do not seem advisable, but other 
+#'   \code{p.adjust} methods are available if you can make sense of them.]
 #'
 #' @section Tests of significance, nonsuperiority, noninferiority, or equivalence:
 #'   When \code{delta = 0}, test statistics are the usual tests of significance.
@@ -332,10 +358,22 @@
 #' # Consider as some of many possible contrasts among the six cell means
 #' summary(con, infer = c(TRUE, TRUE), adjust = "scheffe", scheffe.rank = 5)
 #'
-summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
-                        null, delta, side, frequentist, 
-                        bias.adjust = get_emm_option("back.bias.adj"),
-                        sigma, ...) {
+#' # Show estimates to more digits
+#' print(test(con), digits = 7)
+#' 
+#' # --------------------------------------------------------------
+#' # Cross-adjusting P values
+#' prs <- pairs(warp.emm)   # pairwise comparisons of tension, by wool
+#' test(prs, adjust = "tukey", cross.adjust = "bonferroni")
+#' 
+#' # Same comparisons taken as one big family (more conservative)
+#' test(prs, adjust = "bonferroni", by = NULL)
+#' 
+summary.emmGrid <- function(object, infer, level, adjust, by, 
+                            cross.adjust = "none", type, df, calc,
+                            null, delta, side, frequentist, 
+                            bias.adjust = get_emm_option("back.bias.adj"),
+                            sigma, ...) {
     if(missing(sigma))
         sigma = object@misc$sigma
     if(missing(frequentist) && !is.null(object@misc$frequentist))
@@ -387,8 +425,18 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
         type = .validate.type(type)
     
     # if there are two transformations and we want response, then we need to undo both
-    if ((type == "response") && (!is.null(misc$tran2)))
+    if ((type == "response") && (!is.null(misc$tran2))) {
+        tmp = match.call()
+        tmp$type = "unlink"
+        summ.unlink = eval(tmp) #this is summary with type = "unlink"
         object = regrid(object, transform = "mu")
+        two.trans = TRUE
+    }
+    else
+        two.trans = FALSE
+    
+
+    
     if ((type %in% c("mu", "unlink")) && (!is.null(t2 <- misc$tran2))) {
         if (!is.character(t2))
             t2 = "tran"
@@ -518,7 +566,14 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
                 " so it is omitted")
         }
     }
-
+    
+    
+    # add linkname attribute
+    if (two.trans)
+        linkname = paste0(link$name, "[", attr(summ.unlink, "linkname"), "]")
+    else
+        linkname = link$name
+    
     if(infer[1]) { # add CIs
         acv = .adj.critval(level, result$df, adjust, fam.info, side, corrmat, by.rows, sch.rank)
         ###adjust = acv$adjust # in older versions, I forced same adj method for tests
@@ -533,8 +588,13 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
                 .fmt.sigma(sigma)))
             estName = names(result)[1] = "prediction"
         }
-        result[[cnm[1]]] = result[[1]] + cv[, 1]*result$SE
-        result[[cnm[2]]] = result[[1]] + cv[, 2]*result$SE
+        if (!two.trans) {
+            result[[cnm[1]]] = result[[1]] + cv[, 1]*result$SE
+            result[[cnm[2]]] = result[[1]] + cv[, 2]*result$SE
+        }
+        else {
+            result[cnm] = summ.unlink[cnm]
+        }
         mesg = c(mesg, paste("Confidence level used:", level), acv$mesg)
         if (inv) {
             clims = with(link, cbind(linkinv(result[[cnm[1]]]), linkinv(result[[cnm[2]]])))
@@ -543,37 +603,68 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
             idx = if (all(tmp >= 0)) 1:2 else 2:1
             result[[cnm[1]]] = clims[, idx[1]]
             result[[cnm[2]]] = clims[, idx[2]]
-            mesg = c(mesg, paste("Intervals are back-transformed from the", link$name, "scale"))
+            mesg = c(mesg, paste("Intervals are back-transformed from the", linkname, "scale"))
         }
     }
     if(infer[2]) { # add tests
-        result[["null"]] = null
-        if (inv && !is.null(link))
-            result[["null"]] = link$linkinv(null)
-        if (all(result$null == 0))
-            result[["null"]] = NULL
         tnm = ifelse (zFlag, "z.ratio", "t.ratio")
         tail = ifelse(side == 0, -sign(abs(delta)), side)
-        if (side == 0) {
-            if (delta == 0) # two-sided sig test
-                t.ratio = result[[tnm]] = (result[[1]] - null) / result$SE
-            else
-                t.ratio = result[[tnm]] = (abs(result[[1]] - null) - delta) / result$SE
+        if(!two.trans) {
+            result[["null"]] = null
+            if (inv && !is.null(link))
+                result[["null"]] = link$linkinv(null)
+            if (all(result$null == 0))
+                result[["null"]] = NULL
+            if (side == 0) {
+                if (delta == 0) # two-sided sig test
+                    t.ratio = result[[tnm]] = (result[[1]] - null) / result$SE
+                else
+                    t.ratio = result[[tnm]] = (abs(result[[1]] - null) - delta) / result$SE
+            }
+            else {
+                t.ratio = result[[tnm]] = (result[[1]] - null + side * delta) / result$SE            
+            }
+            apv = .adj.p.value(t.ratio, result$df, adjust, fam.info, tail, corrmat, by.rows, sch.rank)
+            adjust = apv$adjust   # in case it was abbreviated
+            result$p.value = apv$pval
+            mesg = c(mesg, apv$mesg)
         }
         else {
-            t.ratio = result[[tnm]] = (result[[1]] - null + side * delta) / result$SE            
+            result$null = ifelse(is.null(summ.unlink$null), link$linkinv(0), link$linkinv(summ.unlink$null))
+            result[[tnm]] = summ.unlink[[tnm]]
+            result$p.value = summ.unlink$p.value
+            apv = .adj.p.value(0, result$df, adjust, fam.info, tail, corrmat, by.rows, sch.rank)
+            # we ignore everything about apv except the message
+            mesg = c(mesg, apv$mesg)
         }
-        apv = .adj.p.value(t.ratio, result$df, adjust, fam.info, tail, corrmat, by.rows, sch.rank)
-        adjust = apv$adjust   # in case it was abbreviated
-        result$p.value = apv$pval
-        mesg = c(mesg, apv$mesg)
+        
+        # Handle cross-adjustments
+        if ( (length(by.rows) > 1) && 
+             (length(len <- sapply(by.rows, length)) > 1) &&
+             is.na(pmatch(cross.adjust, "none")) ) {
+            val = c("sidak", p.adjust.methods)
+            w = pmatch(tolower(cross.adjust), tolower(val))
+            if (!is.na(w)) {
+                cross.adjust = val[w]
+                mat = matrix(result$p.value, nrow = len)
+                apv = apply(mat, 1, function(p) {
+                    if (w > 1)   p.adjust(p, cross.adjust)
+                    else         1 - (1 - p)^ncol(mat)
+                })
+                result$p.value = as.numeric(t(apv))
+                mesg = c(mesg, paste("Cross-group P-value adjustment:", cross.adjust))
+            }
+            else
+                message("Invalid cross-adjustment method: '", cross.adjust, "'")
+        }
+        
         if (delta > 0)
             mesg = c(mesg, paste("Statistics are tests of", c("nonsuperiority","equivalence","noninferiority")[side+2],
                                  "with a threshold of", signif(delta, 5)))
         if(tail != 0) 
             mesg = c(mesg, paste("P values are ", ifelse(tail<0,"left-","right-"),"tailed", sep=""))
         if (inv) 
-            mesg = c(mesg, paste("Tests are performed on the", link$name, "scale"))
+            mesg = c(mesg, paste("Tests are performed on the", linkname, "scale"))
     }
     if (inv) {
         result[["SE"]] = with(link, abs(mu.eta(result[[1]]) * result[["SE"]]))
@@ -601,6 +692,7 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df, calc,
     attr(summ, "delta") = delta
     attr(summ, "type") = type
     attr(summ, "mesg") = unique(mesg)
+    attr(summ, "linkname") = linkname
     class(summ) = c("summary_emm", "data.frame")
     summ
 }
@@ -688,9 +780,6 @@ predict.emmGrid <- function(object, type,
 #'   \code{emm_options(opt.digits = FALSE)}.
 #' @method as.data.frame emmGrid
 #' @export
-#' @examples
-#' # Show estimates to more digits
-#' print(test(con), digits = 7)
 as.data.frame.emmGrid = function(x, row.names = NULL, optional, check.names = TRUE, ...) {
     as.data.frame(summary(x, ...), row.names = row.names, check.names = check.names)
 }
@@ -841,7 +930,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional, check.names = TR
     et = as.numeric(fam.info[3])
 
     # Force no adjustment when just one test, unless we're using scheffe
-    if ((n.contr == 1) && (adjust %.pin% "scheffe"))  ##(pmatch(adjust, "scheffe", 0) != 1)) 
+    if ((n.contr == 1) && !(adjust %.pin% "scheffe"))  ##(pmatch(adjust, "scheffe", 0) != 1)) 
         adjust = "none"
     
     # do a pmatch of the adjust method
@@ -940,7 +1029,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional, check.names = TR
     et = as.numeric(fam.info[3])
     
     ragged.by = (is.character(fam.size))   # flag that we need to do groups separately
-    if (!ragged.by && (n.contr == 1) && (adjust %.pin% "scheffe"))  ##(pmatch(adjust, "scheffe", 0) != 1)) # Force no adjustment when just one interval, unless using Scheffe
+    if (!ragged.by && (n.contr == 1) && !(adjust %.pin% "scheffe"))  ##(pmatch(adjust, "scheffe", 0) != 1)) # Force no adjustment when just one interval, unless using Scheffe
         adjust = "none"
     
     adj.meths = c("sidak", "tukey", "scheffe", "dunnettx", "mvt", "bonferroni", "none")
