@@ -73,11 +73,14 @@ register_s3_method = function(pkg, generic, class, envir = parent.frame()) {
     if (.requireNS("coda", fail = .nothing)) {
         register_s3_method("coda", "as.mcmc", "emmGrid")
         register_s3_method("coda", "as.mcmc.list", "emmGrid")
+        register_s3_method("coda", "as.mcmc", "emm_list")
+        register_s3_method("coda", "as.mcmc.list", "emm_list")
     }
     if (.requireNS("multcomp", fail = .nothing)) {
         register_s3_method("multcomp", "glht", "emmlf")
         register_s3_method("multcomp", "glht", "emmGrid")
         register_s3_method("multcomp", "cld", "emmGrid")
+        register_s3_method("multcomp", "cld", "emm_list")
         register_s3_method("multcomp", "modelparm", "emmwrap")
     }
 }
